<?php
/**
 * @package Formidable
 */
 
if(!defined('ABSPATH')) die(__('You are not allowed to call this page directly.', 'formidable'));

if(class_exists('FrmEntriesController'))
    return;

class FrmEntriesController{
    
    function FrmEntriesController(){
        add_action('admin_menu', 'FrmEntriesController::menu', 11);
        add_action('wp', 'FrmEntriesController::process_entry', 10, 0);
        add_action('frm_wp', 'FrmEntriesController::process_entry', 10, 0);
        add_filter('frm_redirect_url', 'FrmEntriesController::delete_entry_before_redirect', 50, 3);
        add_action('frm_after_entry_processed', 'FrmEntriesController::delete_entry_after_save', 100);
        add_filter('frm_email_value', 'FrmEntriesController::filter_email_value', 10, 3);
    }
    
    public static function menu(){
        global $frm_vars;
        if(!$frm_vars['pro_is_installed']){
            add_submenu_page('formidable', 'Formidable |'. __('Entries', 'formidable'), '<span style="opacity:.5;filter:alpha(opacity=50);">'. __('Entries', 'formidable') .'</span>', 'administrator', 'formidable-entries', 'FrmEntriesController::list_entries');
        }
    }
    
    public static function list_entries(){
        global $frm_entry;
        $frm_form = new FrmForm();
        $form_select = $frm_form->getAll("is_template=0 AND (status is NULL OR status = '' OR status = 'published')", ' ORDER BY name');
        $form_id = FrmAppHelper::get_param('form', false);
        if($form_id)
            $form = $frm_form->getOne($form_id);
        else
            $form = (isset($form_select[0])) ? $form_select[0] : 0;
        
        if($form)
            $entry_count = $frm_entry->getRecordCount($form->id);
            
        include(FrmAppHelper::plugin_path() .'/classes/views/frm-entries/list.php');
    }
    
    public static function show_form($id='', $key='', $title=false, $description=false){
        global $user_ID, $frm_settings, $post;
        
        $frm_form = new FrmForm();
        if ($id) $form = $frm_form->getOne((int)$id);
        else if ($key) $form = $frm_form->getOne($key);
        else $form = false;

        $form = apply_filters('frm_pre_display_form', $form);
        
        if(!$form or 
            (($form->is_template or $form->status == 'draft') and !isset($_GET) and !isset($_GET['form']) and 
                (!isset($_GET['preview']) or $post and $post->ID != $frm_settings->preview_page_id))
            ){
            return __('Please select a valid form', 'formidable');
        }else if ($form->logged_in and !$user_ID){
            global $frm_settings;
            return do_shortcode($frm_settings->login_msg);
        }

        if($form->logged_in and $user_ID and isset($form->options['logged_in_role']) and $form->options['logged_in_role'] != ''){
            if(FrmAppHelper::user_has_permission($form->options['logged_in_role'])){
                return FrmEntriesController::get_form(FrmAppHelper::plugin_path() .'/classes/views/frm-entries/frm-entry.php', $form, $title, $description);
            }else{
                global $frm_settings;
                return do_shortcode($frm_settings->login_msg);
            }
        }else    
            return FrmEntriesController::get_form(FrmAppHelper::plugin_path() .'/classes/views/frm-entries/frm-entry.php', $form, $title, $description);
    }
    
    public static function get_form($filename, $form, $title, $description) {
        if (is_file($filename)) {
            ob_start();
            include $filename;
            $contents = ob_get_contents();
            ob_end_clean();
            // TODO: check if minimizing is turned on
            //$contents = preg_replace('(\r|\n|\t)', '', $contents);
            return $contents;
        }
        return false;
    }
    
    public static function process_entry($errors='', $ajax=false){
        if((is_admin() and !defined('DOING_AJAX')) or !isset($_POST) or !isset($_POST['form_id']) or !is_numeric($_POST['form_id']) or !isset($_POST['item_key']))
            return;

        global $frm_entry, $frm_vars;
        
        $frm_form = new FrmForm();
        $form = $frm_form->getOne($_POST['form_id']);
        if(!$form)
            return;
        
        $params = FrmEntriesController::get_params($form);
        
        if(!isset($frm_vars['form_params']))
            $frm_vars['form_params'] = array();
        $frm_vars['form_params'][$form->id] = $params;
        
        if(isset($frm_vars['created_entries'][$_POST['form_id']]))
            return;
           
        if($errors == '')
            $errors = $frm_entry->validate($_POST);
        $frm_vars['created_entries'][$_POST['form_id']] = array('errors' => $errors);
        
        if( empty($errors) ){
            $_POST['frm_skip_cookie'] = 1;
            if($params['action'] == 'create'){
                if (apply_filters('frm_continue_to_create', true, $_POST['form_id']) and !isset($frm_vars['created_entries'][$_POST['form_id']]['entry_id']))
                    $frm_vars['created_entries'][$_POST['form_id']]['entry_id'] = $frm_entry->create( $_POST );
            }
            
            do_action('frm_process_entry', $params, $errors, $form, array('ajax' => $ajax));
            unset($_POST['frm_skip_cookie']);
        }
    }
    
    public static function delete_entry_before_redirect($url, $form, $atts){
        self::_delete_entry($atts['id'], $form);
        return $url;
    }
    
    //Delete entry if not redirected
    public static function delete_entry_after_save($atts){
        self::_delete_entry($atts['entry_id'], $atts['form']);
    }
    
    private static function _delete_entry($entry_id, $form){
        if(!$form)
            return;
        
        $form->options = maybe_unserialize($form->options);
        if(isset($form->options['no_save']) and $form->options['no_save']){
            global $frm_entry;
            $frm_entry->destroy( $entry_id );
        }
    }
    
    public static function &filter_email_value($value, $meta, $entry, $atts=array()){
        $frm_field = new FrmField();
        $field = $frm_field->getOne($meta->field_id);
        if(!$field)
            return $value; 
            
        $value = self::filter_display_value($value, $field, $atts);
        return $value;
    }
    
    public static function &filter_display_value($value, $field, $atts=array()){
        $field->field_options = maybe_unserialize($field->field_options);
        
        $saved_value = (isset($atts['saved_value']) and $atts['saved_value']) ? true : false;
        if(!in_array($field->type, array('radio', 'checkbox', 'radio', 'select')) or !isset($field->field_options['separate_value']) or !$field->field_options['separate_value'] or $saved_value)
            return $value;
            
        $field->options = maybe_unserialize($field->options);
        $f_values = array();
        $f_labels = array();
        foreach($field->options as $opt_key => $opt){
            if(!is_array($opt))
                continue;
            
            $f_labels[$opt_key] = isset($opt['label']) ? $opt['label'] : reset($opt);
            $f_values[$opt_key] = isset($opt['value']) ? $opt['value'] : $f_labels[$opt_key];
            if($f_labels[$opt_key] == $f_values[$opt_key]){
                unset($f_values[$opt_key]);
                unset($f_labels[$opt_key]);
            }
            unset($opt_key);
            unset($opt);
        }

        if(!empty($f_values)){
            foreach((array)$value as $v_key => $val){
                if(in_array($val, $f_values)){
                    $opt = array_search($val, $f_values);
                    if(is_array($value))
                        $value[$v_key] = $f_labels[$opt];
                    else
                        $value = $f_labels[$opt];
                }
                unset($v_key);
                unset($val);
            }
        }
        
        return $value;
    }
    
    public static function get_params($form=null){
        global $frm_vars;
        
        $frm_form = new FrmForm();
        if(!$form)
            $form = $frm_form->getAll(array(), 'name', 1);
        else if(!is_object($form))
            $form = $frm_form->getOne($form);
        
        if(isset($frm_vars['form_params']) && is_array($frm_vars['form_params']) && isset($frm_vars['form_params'][$form->id]))
            return $frm_vars['form_params'][$form->id];
           
        $action_var = isset($_REQUEST['frm_action']) ? 'frm_action' : 'action';
        $action = apply_filters('frm_show_new_entry_page', FrmAppHelper::get_param($action_var, 'new'), $form);
        
        $default_values = array(
            'id' => '', 'form_name' => '', 'paged' => 1, 'form' => $form->id, 'form_id' => $form->id, 
            'field_id' => '', 'search' => '', 'sort' => '', 'sdir' => '', 'action' => $action
        );
            
        $values['posted_form_id'] = FrmAppHelper::get_param('form_id');
        if (!is_numeric($values['posted_form_id']))
            $values['posted_form_id'] = FrmAppHelper::get_param('form');

        if ($form->id == $values['posted_form_id']){ //if there are two forms on the same page, make sure not to submit both
            foreach ($default_values as $var => $default){
                if($var == 'action')
                    $values[$var] = FrmAppHelper::get_param($action_var, $default);
                else
                    $values[$var] = FrmAppHelper::get_param($var, $default);
                unset($var);
                unset($default);
            }
        }else{
            foreach ($default_values as $var => $default){
                $values[$var] = $default;
                unset($var);
                unset($default);
            }
        }

        if(in_array($values['action'], array('create', 'update')) and (!isset($_POST) or (!isset($_POST['action']) and !isset($_POST['frm_action']))))
            $values['action'] = 'new';

        return $values;
    }
    
}
